==> json-joy-2.8.0.tgz: package/esm/json-crdt/codec/json/Encoder.js <==
import { ConstantType } from '../../types/const/ConstantType';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringType } from '../../types/rga-string/StringType';
export class Encoder {
    encode(doc) {
        const snapshot = {
            clock: this.encodeClock(doc.clock),
            root: this.encodeRoot(doc.root),
        };
        return snapshot;
    }
    encodeClock(clock) {
        const data = [];
        for (const c of clock.clocks.values())
            data.push([c.sessionId, c.time]);
        return data;
    }
    encodeTimestamp(ts) {
        return [ts.sessionId, ts.time];
    }
    encodeRoot(root) {
        return {
            type: 'root',
            id: this.encodeTimestamp(root.id),
            node: root.node ? this.encodeNode(root.node) : null,
        };
    }
    encodeNode(node) {
        if (node instanceof ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType)
            return this.encodeConst(node);
        throw new Error('UNKNOWN_NODE');
    }
    encodeObj(obj) {
        const chunks = {};
        for (const [key, entry] of obj.latest.entries())
            chunks[key] = {
                id: this.encodeTimestamp(entry.id),
                node: this.encodeNode(entry.node),
            };
        return {
            type: 'obj',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeArrChunk(chunk));
        return {
            type: 'arr',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            nodes: chunk.nodes.map(n => this.encodeNode(n)),
        };
        return res;
    }
    encodeStr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeStrChunk(chunk));
        return {
            type: 'str',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeStrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            value: chunk.str,
        };
        return res;
    }
    encodeVal(obj) {
        return {
            type: 'val',
            id: this.encodeTimestamp(obj.id),
            writeId: this.encodeTimestamp(obj.writeId),
            value: obj.value,
        };
    }
    encodeConst(obj) {
        return {
            type: 'const',
            value: obj.value,
        };
    }
}

==> json-joy-7.6.0.tgz: package/esm/json-crdt/codec/json/Encoder.js <==
import { ArrayType } from '../../types/rga-array/ArrayType';
import { BinaryType } from '../../types/rga-binary/BinaryType';
import { ConstantType } from '../../types/const/ConstantType';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { StringType } from '../../types/rga-string/StringType';
import { toBase64 } from '../../../util/base64/encode';
import { ValueType } from '../../types/lww-value/ValueType';
export class Encoder {
    encode(model) {
        const clock = model.clock;
        const isServerClock = clock.getSessionId() === 1;
        return {
            time: isServerClock ? clock.time : this.encodeClock(model.clock),
            root: this.encodeRoot(model.root),
        };
    }
    encodeClock(clock) {
        const data = [];
        const sessionId = clock.getSessionId();
        const localTs = clock.clocks.get(sessionId);
        if (!localTs)
            data.push([sessionId, clock.time]);
        for (const c of clock.clocks.values())
            data.push([c.getSessionId(), c.time]);
        return data;
    }
    encodeTimestamp(ts) {
        return ts.getSessionId() === 1
            ? ts.time
            : [ts.getSessionId(), ts.time];
    }
    encodeRoot(root) {
        return {
            type: 'root',
            id: this.encodeTimestamp(root.id),
            node: root.node ? this.encodeNode(root.node) : null,
        };
    }
    encodeNode(node) {
        if (node instanceof ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType)
            return this.encodeConst(node);
        else if (node instanceof BinaryType)
            return this.encodeBin(node);
        throw new Error('UNKNOWN_NODE');
    }
    encodeObj(obj) {
        const chunks = {};
        for (const [key, entry] of obj.latest.entries())
            chunks[key] = {
                id: this.encodeTimestamp(entry.id),
                node: this.encodeNode(entry.node),
            };
        return {
            type: 'obj',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeArrChunk(chunk));
        return {
            type: 'arr',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            nodes: chunk.nodes.map((n) => this.encodeNode(n)),
        };
        return res;
    }
    encodeStr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeStrChunk(chunk));
        return {
            type: 'str',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeStrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            value: chunk.str,
        };
        return res;
    }
    encodeBin(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeBinChunk(chunk));
        return {
            type: 'bin',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeBinChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            value: toBase64(chunk.buf),
        };
        return res;
    }
    encodeVal(obj) {
        return {
            type: 'val',
            id: this.encodeTimestamp(obj.id),
            writeId: this.encodeTimestamp(obj.writeId),
            value: obj.value,
        };
    }
    encodeConst(obj) {
        return {
            type: 'const',
            value: obj.value,
        };
    }
}
